local _G = GLOBAL
local TheNet = _G.TheNet
if not(TheNet and TheNet: GetIsServer()) then return end

local SpawnPrefab = _G.SpawnPrefab
local STRINGS = _G.STRINGS
local WorldState

local announce_tip = GetModConfigData("ANNOUNCE_TIP")
local packs_cd = GetModConfigData("PACKS_CD")
local resource_balance = GetModConfigData("RESOURCE_BALANCE")
local packs_character = GetModConfigData("PACKS_CHARACTER")
local packs_build = GetModConfigData("PACKS_BUILD")
local packs_season = GetModConfigData("PACKS_SEASON")
local packs_buff = GetModConfigData("PACKS_BUFF") * 60
local packs_science = GetModConfigData("PACKS_SCIENCE")
local packs_backpack = GetModConfigData("PACKS_BACKPACK")

local MY_STRINGS = {
    season_name = {
        "Spring",
        "Late Spring",
        "Summer",
        "Late Summer",
        "Autumn",
        "Late Autumn",
        "Winter",
        "Late Winter",
    },
    get_packs_msg = "%s (%s) get a < %s NovicePacks > CD:%d days",
    get_one_packs_msg = "%s (%s) get a < %s NovicePacks >",
    get_build_packs_msg = "%s (%s) get a < Manufacture NovicePacks >",
    wait_packs_msg = "%s , < NovicePacks > CD:%d days",
}

do
    -- auto load languages translate
    local support_languages = { chs = true, cht = true, zh_CN = "chs", TW = "cht", }
    local steam_support_languages = { schinese = "chs", tchinese = "cht", }

    AddPrefabPostInit("world",function(inst)
        local steamlang = TheNet:GetLanguageCode() or nil
        if steamlang and steam_support_languages[steamlang] then
            print("<Starting NovicePacks> Get your language from steam!")
            modimport("translate_"..steam_support_languages[steamlang])
        else
            local lang = _G.LanguageTranslator.defaultlang or nil
            if lang ~= nil and support_languages[lang] ~= nil then
                if support_languages[lang] ~= true then
                    lang = support_languages[lang]
                end
                print("<Starting NovicePacks> Get your language from language mod!")
                modimport("translate_"..lang)
            end
        end

        if MY_STRINGS_OVERRIDE ~= nil then
            for k,v in pairs(MY_STRINGS_OVERRIDE) do
                if MY_STRINGS[k] ~= nil then
                    MY_STRINGS[k] = v
                end
            end
        end

    end)
end

local resource = {}
do
    modimport("modinfo")
    for k, v in pairs(resource_prefab) do
        resource[v.name] = GetModConfigData(v.name)
    end
    server_filter_tags = nil
    description = nil
    configuration_options = nil
    default_number = nil
    resource_number = nil
    resource_prefab = nil
end

local cd_list = {}
local build_list = {number=0,}

local season_item = {
    {
        umbrella = 1,
        raincoat = 1,
    },
    {
        umbrella = 1,
        heatrock = 1,
        ice = 5,
    },
    {
        watermelonhat = 1,
        heatrock = 1,
        ice = 15,
    },
    {
        heatrock = 1,
        ice = 10,
        dragonpie = 2,
    },
    {
        sewing_kit = 1,
        dragonpie = 5,
    },
    {
        heatrock = 1,
        dragonpie = 5,
    },
    {
        heatrock = 1,
        winterhat = 1,
    },
    {
        umbrella = 1,
        heatrock = 1,
    },
}

local character_item = {
    wilson = {
        razor = 1,
        twigs = 10,
        cutgrass = 10,
    },
    willow = {
        flowerhat = 1,
        tophat = 1,
        spear = 1,
    },
    wendy = {
        armorwood = 1,
        tophat = 1,
    },
    woodie = {
        log = 5,
        pinecone = 5,
        footballhat = 1,
    },
    wolfgang = {
        hambat = 1,
        footballhat = 1,
    },
    wx78 = {
        gears = 5,
        umbrella = 1,
    },
    wickerbottom = {
        honeycomb = 1,
        cactus_meat = 5,
        papyrus = 2,
    },
    wes = {
        goldnugget = 10,
        fireflies = 1,
    },
    waxwell = {
        nightmarefuel = 5,
        papyrus = 2,
    },
    wathgrithr = {
        goldnugget = 5,
        rocks = 5,
    },
    webber = {
        razor = 1,
        spidereggsack = 1,
    },
}
local character_backpack = {
    wilson = 1,
    willow = 1,
    wendy = 1,
    woodie = 1,
    wolfgang = 0,
    wx78 = 1,
    wickerbottom = 1,
    wes = 2,
    waxwell = 0,
    wathgrithr = 0,
    webber = 0,
}
local maxbase = 5

local function ItemBalance(count)
    local base = WorldState.cycles * 0.03 + 1
    if base > maxbase then
        base = maxbase
    end
    return math.floor(base * count)
end

local function GiveResource(player, prefab, count)
    local inventory = player and player.components.inventory or nil
    local lastitem = nil
    for i = 1, count do
        local item = SpawnPrefab(prefab)
        if item == nil then
            return
        elseif resource_balance and i == 1 and item.components and item.components.equippable == nil and item.components.inventoryitem then
            count = ItemBalance(count)
        end
        -- heatrock heating / refrigeration
        if prefab == "heatrock" then
            if WorldState.issummer or WorldState.isspring and WorldState.remainingdaysinseason < 3 then
                item.components.temperature: DoDelta(-80)
            elseif WorldState.iswinter or WorldState.isautumn and WorldState.remainingdaysinseason < 3 then
                item.components.temperature: DoDelta(80)
            end
        end

        local isgive = false
        if item.components and item.components.equippable then
            local itemequipslot = item.components.equippable.equipslot
            local itemequip = inventory: GetEquippedItem(itemequipslot)
            if itemequip == nil then
                inventory: Equip(item)
                isgive = true
            end
        end

        if not isgive then
            local gived = inventory: GiveItem(item)
            if gived == nil then
                if lastitem then
                    lastitem: Remove()
                    lastitem = nil
                end
                return
            else
                lastitem = gived == true and item or nil
            end
        end
        player.novicepacks.ispacks = true
    end
end

local function UnlockScience(player)
    if packs_science == 1 then
        player.components.builder:UnlockRecipesForTech({SCIENCE = 1})
    elseif packs_science == 2 then
        player.components.builder:UnlockRecipesForTech({SCIENCE = 2})
    elseif packs_science == 3 then
        player.components.builder:UnlockRecipesForTech({SCIENCE = 2, MAGIC = 2})
    elseif packs_science >= 4 then
        player.components.builder:UnlockRecipesForTech({SCIENCE = 2, MAGIC = 3})
    end
end

local function AddBuff(player)
    local initstate = {}
    initstate.absorb = player.components.health.absorb
    initstate.maxtemp = player.components.temperature.maxtemp
    initstate.mintemp = player.components.temperature.mintemp
    -- damage absorption
    player.components.health.absorb = 0.8
    -- constant temperature
    player.components.temperature.maxtemp = 40
    player.components.temperature.mintemp = 20
    player: AddTag("forcefield")
    local fx = SpawnPrefab("forcefieldfx")
    fx.entity: SetParent(player.entity)
    fx.Transform: SetPosition(0, 0.2, 0)
    local fx_hitanim = function()
        fx.AnimState: PlayAnimation("hit")
        fx.AnimState: PushAnimation("idle_loop")
    end
    fx: ListenForEvent("blocked", fx_hitanim, player)
    player.active = true

    player: DoTaskInTime(packs_buff, function()
        fx: RemoveEventCallback("blocked", fx_hitanim, player)
        fx.kill_fx(fx)
        if player: IsValid() then
            -- Return to normal state.
            player.components.health.absorb = initstate.absorb
            player.components.temperature.maxtemp = initstate.maxtemp
            player.components.temperature.mintemp = initstate.mintemp
            player: RemoveTag("forcefield")
            player: DoTaskInTime(1, function() player.active = nil end)
        end
    end)
end

local function GetSeason()
    if WorldState.isspring then
        return 1
    elseif WorldState.issummer then
        return 2
    elseif WorldState.isautumn then
        return 3
    elseif WorldState.iswinter then
        return 4
    end
end

local OnPlayerSpawn = function(src, player)
    if packs_science == 5 then
        player.components.builder.ancient_bonus = 4
    end

    local Old_OnNewSpawn = player.OnNewSpawn
    player.OnNewSpawn = function(...)

        player.novicepacks = {}
        if packs_cd == 0 or cd_list[player.userid] == nil or(packs_cd > 0 and cd_list[player.userid] + packs_cd <= WorldState.cycles) then
            -- Close SpawnPrefab sound.
            player.components.inventory.ignoresound = true
            local isbuild = false
            if packs_science >= 1 and packs_science <= 5 then
                UnlockScience(player)
                player.novicepacks.ispacks = true
            end
            if packs_buff ~= 0 then
                AddBuff(player)
                player.novicepacks.ispacks = true
            end
            if packs_build > 0 and WorldState.cycles <= 10 and build_list.number < packs_build and build_list[player.userid] == nil and player.components and player.components.builder and math.random() < 0.7 then
                player.components.builder: GiveAllRecipes()
                local Old_RemoveIngredients = player.components.builder.RemoveIngredients
                player.components.builder.RemoveIngredients = function(self, inst, recname)
                    player.components.builder.RemoveIngredients = Old_RemoveIngredients
                    Old_RemoveIngredients(self, inst, recname)
                    player.components.builder: UnlockRecipe(recname)
                    player.components.builder: GiveAllRecipes()
                end
                build_list.number = build_list.number + 1
                build_list[player.userid] = true
                player.novicepacks.ispacks = true
                isbuild = true
            end
            local backpack_type = packs_backpack
            if packs_character and character_backpack[player.prefab] and character_backpack[player.prefab] > backpack_type then
                backpack_type = character_backpack[player.prefab]
            end
            local backpacks = {
                "backpack",
                "piggyback",
                "icepack",
                "krampus_sack",
            }
            if backpacks[backpack_type] then
                player.components.inventory: Equip(SpawnPrefab(backpacks[backpack_type]))
                player.novicepacks.ispacks = true
            end
            local resources = {}
            local season_msg = ""
            for k, v in pairs(resource) do
                resources[k] = v
            end
            if packs_character and character_item[player.prefab] then
                for k, v in pairs(character_item[player.prefab]) do
                    if resources[k] == nil or resources[k] < v then
                        resources[k] = v
                    end
                end
            end
            if packs_season then
                local season_type = GetSeason() * 2
                if WorldState.remainingdaysinseason >= 3 then
                    season_type = season_type - 1
                end
                season_msg = MY_STRINGS.season_name[season_type]
                for k, v in pairs(season_item[season_type]) do
                    resources[k] = resources[k] and resources[k] + v or v
                end
            end
            for k, v in pairs(resources) do
                GiveResource(player, k, v)
            end

            if player.novicepacks.ispacks then
                cd_list[player.userid] = WorldState.cycles
                if announce_tip then
                    local charactername = STRINGS.CHARACTER_NAMES[player.prefab] or player.prefab
                    local tip = packs_cd > 0 and string.format(MY_STRINGS.get_packs_msg, player: GetDisplayName(), charactername, season_msg, packs_cd)
                        or string.format(MY_STRINGS.get_one_packs_msg, player: GetDisplayName(), charactername, season_msg)
                    player: DoTaskInTime(0.5, function() TheNet: Announce(tip) end)
                    if isbuild then
                        player: DoTaskInTime(1, function() TheNet: Announce(string.format(MY_STRINGS.get_build_packs_msg, player: GetDisplayName(), charactername)) end)
                    end
                end
            end
            player.components.inventory.ignoresound = false

        elseif packs_cd < 0 then
        else
            local tip = string.format(MY_STRINGS.wait_packs_msg, player: GetDisplayName(), (cd_list[player.userid] + packs_cd - WorldState.cycles))
            player: DoTaskInTime(1, function() if player.components.talker then player.components.talker: Say(tip) end end)
        end

        player.novicepacks.ispacks = nil
        player.novicepacks = nil

        if Old_OnNewSpawn then
            return Old_OnNewSpawn(...)
        end
    end
end

local OldOnSave, OldOnLoad
local function OnSave(inst, data)
    if OldOnSave ~= nil then
        OldOnSave(inst, data)
    end
    if cd_list ~= nil and build_list ~= nil then
        data.novicepacks = {
            ["cd_list"] = cd_list,
            ["build_list"] = build_list,
        }
    end
end

local function OnLoad(inst, data)
    if OldOnLoad ~= nil then
        OldOnLoad(inst, data)
    end
    if data ~= nil and data.novicepacks ~= nil then
        cd_list = data.novicepacks.cd_list
        build_list = data.novicepacks.build_list
    end
end

AddPrefabPostInit("world", function(inst)
    WorldState = _G.TheWorld.state
    OldOnSave = inst.OnSave
    inst.OnSave = OnSave
    OldOnLoad = inst.OnLoad
    inst.OnLoad = OnLoad
    inst: ListenForEvent("ms_playerspawn", OnPlayerSpawn, inst)
end)
